/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef LETTER_H_
#define LETTER_H_
#include <QList>
#include <QString>
#include <QTableWidgetItem>
#include "Collation.h"
#include "tools/UCTools.h"
#include <QHash>
#include <QBrush>
#include "tools/UCTools.h"
#include <QFontMetrics>
#include <QDebug>

/**
 * \class Letter
 * \brief Data class for Unicode letters.
 *
 * Container for Unicode letters with the attributes codepoint, collation,
 * comment, version and block. The codepoint may consist of multiple uint
 * values. The codepoint is always considered to be UTF-16, but can be
 * received in UTF-8 as a String (getCodepointAsUtf8()). The collations are stored
 * in Collation containers. All attributes are accessible through get and
 * set methods. Letters can be compared with the operator==() and the
 * equals() method. Operator==() only compares the codepoints. Equals() takes all
 * attributes, except version and block, into account. The operator<() method
 * compares only the codepoints first values. For use in a QHash a global
 * qHash() method is provided.
 * Objects of this class are managed in the TableModel class.
 *
 * \author Martin Burgemeister <Martin.Burgemeister@gmx.de>,
 *         Mathis Lindemann <MathisLindemann@gmx.net>
 * \version 1.0
 * \date 16 August 2010
 * \see Collation
 */
class Letter: public QTableWidgetItem
{
public:
    Letter  ();
    Letter  ( const Letter* other );

    QString getComment             () const;
    QString getBlock               () const;
    QString getCodepointAsHex      () const;
    QString getCodepointAsUtf8     () const;
    QString getLinestring          () const;
    QString getLinestringUtf8      () const;
    QString getVersionInfo         () const;
    QList<uint> getCodepoint       () const;
    QList<Collation> getCollations () const;

    void setComment     ( QString );
    void setBlock       ( QString );
    void setVersionInfo ( QString );
    void setCodepoint   ( QList<uint> );
    void setCollations  ( QList<Collation> );

    void addCollation   ( Collation );

    bool operator== ( const Letter& ) const;
    bool equals     ( const Letter& ) const;
    bool operator<  ( const Letter& ) const;

    QBrush background ();

//    bool inFont( const QFont& );

private:
    /**
     * \brief The collations of the letter.
     */
    QList<Collation> collations_;
    /**
     * \brief The comment of the letter.
     */
    QString comment_;
    /**
     * \brief The codepoint of the letter.
     */
    QList<uint> codepoint_;
    /**
     * \brief The version of the letter.
     */
    QString versionInfo_;
    /**
     * \brief The block this letter belongs to.
     */
    QString block_;
};

/**
 * \brief Hash method for the use of Letter in QHash.
 *
 * \param key The letter object to hash.
 * \return The hash.
 */
inline uint qHash ( const Letter& key )
{
    return ::qHash ( key.getCodepointAsHex () );
}

#endif /* LETTER_H_ */
