/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *     																	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef UCQTABLEVIEW_H_
#define UCQTABLEVIEW_H_
#include <QTableView>
#include <QHeaderView>
#include <QFont>
#include <QList>
#include <QStringList>
#include "data/Letter.h"
#include <QResizeEvent>
#include <QScrollBar>
#include <QMouseEvent>
#include "data/TableModel.h"
#include <QClipboard>
#include <QDebug>
/**
 * \class UCQTableView
 * \brief Extends QTableView to provide a little extra functionality.
 *
 * The UCQTableView provides methods to change the appearance of the letters in
 * the table, i.e. their font, the fontsize and bold/italic. It also allows to
 * change the size of the cells by setting a zoomfactor. The most important
 * extension to QTextEdit is the ability to calculate how many columns can be
 * displayed using the current cellsize. On every resize event the number of
 * maximal visible columns without a horizontal scrollbar is calculated and
 * emitted via the signal resized(). That signal is received by the TableModel,
 * which changes the models column- and row- count accordingly and updates the
 * view.
 *
 * \author Martin Burgemeister <Martin.Burgemeister@gmx.de>,
 *         Mathis Lindemann <MathisLindemann@gmx.net>
 * \version 1.0
 * \date 16 August 2010
 * \see UCQDetails
 * \see TableModel
 */
class UCQTableView: public QTableView
{

Q_OBJECT

public:
    UCQTableView ( QWidget* );

    QModelIndexList getSelectedIndexes ();

    /**
     * \brief Empty re-implementation to disable keyboard search.
     */
    void keyboardSearch ( const QString& ){}

protected:
    void mouseMoveEvent ( QMouseEvent*  );
    QModelIndex moveCursor ( CursorAction , Qt::KeyboardModifiers );
private:
    /**
     * \brief Current cell size.
     */
    int cellSize_;
    /**
     * \brief Current zoom factor.
     */
    double zoomFactor_;
    /**
     * \brief Current fontsize of the letters.
     */
    int fontsize_;

public slots:
    void keyPressEvent     ( QKeyEvent*   );
    void mousePressEvent   ( QMouseEvent* );
    void mouseReleaseEvent ( QMouseEvent* );

    void resizeCells     ();
    void clearSelections ();

    void setLetterSelected  ( QModelIndex,            bool );
    void setLettersSelected ( const QModelIndexList&, bool );

    void scrollTo(QModelIndex);

protected slots:
    void changeFont     ( const QFont&   );
    void changeFontSize ( const QString& );

    void setBold   ( bool );
    void setItalic ( bool );

    void zoomIn  ();
    void zoomOut ();
    void resizeEvent ( QResizeEvent* );
    void selectionChanged ( const QItemSelection&, const QItemSelection& );
    void doubleClicked ( QModelIndex );

signals:
    /**
     * \brief Signals a change of the selection.
     *
     * \param selection List of the selected indices.
     */
    void newSelection ( QModelIndexList selection );
    /**
     * \brief Signals that the selected letters should be printed in the
     *        console.
     *
     * \param selection List of the selected Letters.
     */
    void printSelection ( QList<Letter *> selection );
    /**
     * \brief Signals a request to show the details window.
     *
     * \param event The mouse event.
     */
    void showDetails ( QMouseEvent* event );
    /**
     * \brief Signals that the details window should be closed.
     */
    void closeDetails ();
    /**
     * \brief Signals a move of the details window and the new coordinates.
     *
     * \param x The x coordinate to which to move.
     * \param y The y coordinate to which to move.
     */
    void moveDetails ( int x, int y );
    /**
     * \brief Signals a resize and how many columns can be shown without
     *        displaying a horizontal scrollbar.
     *
     * \param columnCount The new column count.
     */
    void resized ( int columnCount );
    /**
     * \brief Signals that the Letter was double clicked.
     *
     * \param letter The clicked letter.
     */
    void doubleClicked ( Letter* letter );
};

#endif /* UCQTABLEVIEW_H_ */
